/*
 * This file is a part of EZC -- Easy templating in C++ library
 * and is distributed under the BSD 3-Clause licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2021, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "models.h"


#ifdef EZC_HAS_MORM_LIBRARY



namespace Ezc
{


Models::Models()
{
}


Models::~Models()
{
	Clear();
}


void Models::Clear()
{
	for(auto & map_item : models_map)
	{
		if( map_item.second->should_be_auto_removed() )
		{
			delete map_item.second;
			map_item.second = nullptr;
		}
	}

	models_map.clear();
}



void Models::Add(const std::wstring & name, morm::Model & model)
{
	morm::ModelWrapper * models_base = new morm::ModelWrapperModel(&model);
	models_map[name] = models_base;
}


void Models::Add(const std::wstring & name, morm::Model * model)
{
	morm::ModelWrapper * models_base = new morm::ModelWrapperModel(model);
	models_map[name] = models_base;
}


morm::ModelWrapper * Models::Find(const std::wstring & name)
{
	auto iterator = models_map.find(name);

	if( iterator != models_map.end() )
	{
		return iterator->second;
	}

	return nullptr;
}



}


#endif

